// Force strict mode and setup for ESM
"use strict";
import {
  require_react
} from "./chunk-KYMBIKIW.js";
import {
  init_esbuild_shims
} from "./chunk-A4BMJM77.js";
import {
  __name,
  __toESM
} from "./chunk-J2S4EL5Y.js";

// packages/cli/src/ui/types.ts
init_esbuild_shims();
var import_react = __toESM(require_react(), 1);
var TERMINAL_GOAL_STATUS_KINDS = [
  "achieved",
  "aborted",
  "failed"
];
function isTerminalGoalStatusKind(kind) {
  return TERMINAL_GOAL_STATUS_KINDS.includes(
    kind
  );
}
__name(isTerminalGoalStatusKind, "isTerminalGoalStatusKind");

// packages/cli/src/ui/commands/types.ts
init_esbuild_shims();

export {
  isTerminalGoalStatusKind
};
/**
 * @license
 * Copyright 2025 Google LLC
 * SPDX-License-Identifier: Apache-2.0
 */
